%{

#include <string.h>
#include <stdlib.h>
#include "ipac.h"
#include "config.h"

#define CONFDEBUG 1

int conferror(char *s);
int confparse (void);
int conflex(void);

extern FILE *confin;
extern char *storage_method_name;
extern char *acc_agent_name;
extern char *access_agent_name;
extern char *rulesfile;
extern int slogin;
extern char *authhost;
extern int dropzero;
extern int confline;

%}

%union {
	char		*s;
	unsigned int	n;
}

%token  <s> PARAMETER
%token  <n> BOOL

%token  EQUAL
%token	CLASSIC ACCESS ACCOUNT STORAGE RULES SLOGIN AUTHHOST DROPZERO ERROR

%%

input:
	    |
	    input commands;

commands:
	    error		{ fprintf(stderr,  "error while parsing config file\n"); yyerrok; 
				    }
	    | command
	    ;

command:
	    classic_opt
	    |access_opt
	    |account_opt
	    |storage_opt
	    |rules_opt
	    |slogin_opt
	    |authhost_opt
	    |dropzero_opt;

classic_opt:
	    CLASSIC EQUAL BOOL
		{
			is_classic = conflval.n;
		}
	    | CLASSIC error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

access_opt:
	    ACCESS EQUAL PARAMETER
		{
			access_agent_name = xstrdup(conflval.s);
		}
	    | ACCESS error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

account_opt:
	    ACCOUNT EQUAL PARAMETER
		{
			acc_agent_name = xstrdup(conflval.s);
		}
	    | ACCOUNT error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

storage_opt:
	    STORAGE EQUAL PARAMETER
		{
			storage_method_name = xstrdup(conflval.s);
		}
	    | STORAGE error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

rules_opt:
	    RULES EQUAL PARAMETER
		{
			rulesfile = xstrdup(conflval.s);
		}
	    | RULES error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

slogin_opt:
	    SLOGIN EQUAL BOOL
		{
			slogin = conflval.n;
		}
	    | SLOGIN error
		{
			fprintf(stderr, "error while parsing config file\n");
		};

authhost_opt:
	    AUTHHOST EQUAL PARAMETER
		{
			authhost = xstrdup(conflval.s);
		}
	    | AUTHHOST error
		{
			fprintf(stderr, "error while parsing config file\n");
		};
		
dropzero_opt:
	    DROPZERO EQUAL BOOL
		{
			dropzero = conflval.n;
		}
	    | DROPZERO error
		{
		    	fprintf(stderr, "error while parsing config file\n");
		};
%%

int 
parse_config(FILE *in) 
{
	confin = (in == NULL) ? stdin : in;
	return confparse();
}

int
conferror(char *s)
{
	fprintf(stderr, "Config parse error near line %d: %s\n", confline, s);
	return -1;
}
